local vars = require('user.config.vars')

-- vim.lsp.set_log_level(vars.loglevel)
require('lspconfig.ui.windows').default_options.border = vars.border
require('user.plugins.lsp.lspconfig.diagnostic').configure_diagnostic()

vim.api.nvim_create_autocmd('LspAttach', {
    group = vim.api.nvim_create_augroup('UserLspConfig', {}),
    callback = function(args)
        local buffer = args.buf
        local client = vim.lsp.get_client_by_id(args.data.client_id)

        -- Key bindings
        require('user.plugins.lsp.lspconfig.keymap').on_attach(client, buffer)

        -- Inlay hints
        -- TODO: uncomment when nvim-0.10.0 is out
        -- if vars.lsp_inlay_hints and client.supports_method('textDocument/inlayHint') then
        --     vim.lsp.inlay_hint.enable(buffer)
        -- end

        -- Bulb for code action hint in statusbar
        if client.supports_method('textDocument/codeAction') then
            vim.api.nvim_create_autocmd({ 'CursorHold', 'InsertLeave' }, {
                group = vim.api.nvim_create_augroup('LspCodeAction', {}),
                callback = require('user.util.lsp').codeAction_on_attach,
                buffer = buffer,
            })
        end

        -- Refresh codelens
        if client.supports_method('textDocument/codeLens') then
            vim.api.nvim_create_autocmd({ 'InsertLeave', 'BufEnter', 'CursorHold' }, {
                group = vim.api.nvim_create_augroup('LspCodeLens', {}),
                callback = vim.lsp.codelens.refresh,
                buffer = buffer,
            })
        end

        -- Document highlight
        if vars.lsp_document_highlight and client.supports_method('textDocument/documentHighlight') then
            local augroup = 'LspDocumentHighlight'

            vim.api.nvim_create_augroup(augroup, {})
            vim.api.nvim_create_autocmd({ 'CursorHold', 'CursorHoldI' }, {
                group = augroup,
                callback = vim.lsp.buf.document_highlight,
                buffer = buffer,
            })
            vim.api.nvim_create_autocmd('CursorMoved', {
                group = augroup,
                callback = vim.lsp.buf.clear_references,
                buffer = buffer,
            })
        end

        -- Prefer hover capability from other LSP servers (pylsp/pyright/pylyzer)
        if client.name == 'ruff_lsp' then
            client.server_capabilities.hoverProvider = false
        end
    end,
})

local servers = require('user.plugins.lsp.lspconfig.servers')
local capabilities = require('cmp_nvim_lsp').default_capabilities(vim.lsp.protocol.make_client_capabilities())
for server, server_opts in pairs(servers) do
    local opts = vim.tbl_deep_extend('force', {
        capabilities = vim.deepcopy(capabilities),
    }, server_opts or {})

    require('lspconfig')[server].setup(opts)
end
