package main

import (
	"fmt"
	"slices"
	"sort"
)

func listPkgs(targets map[string]Target) error {
	// Categorize the list
	categories := map[string][]string{
		"LSP servers":        {},
		"DAP servers":        {},
		"Linters/Formatters": {},
	}

	for target, spec := range targets {
		// NOTE: there will be duplications (items belonging to multiple categories, .e.g elixirls), but it is intentional
		if slices.Contains(spec.Category, "lsp") {
			categories["LSP servers"] = append(categories["LSP servers"], target)
		}
		if slices.Contains(spec.Category, "dap") {
			categories["DAP servers"] = append(categories["DAP servers"], target)
		}
		if slices.Contains(spec.Category, "lint") {
			categories["Linters/Formatters"] = append(categories["Linters/Formatters"], target)
		}
	}

	// Iterate over a list, to make the output stable
	categoryList := []string{}
	for name := range categories {
		categoryList = append(categoryList, name)
	}
	sort.Strings(categoryList)

	for _, name := range categoryList {
		fmt.Printf("* %s:\n", name)
		sort.Strings(categories[name])
		for _, target := range categories[name] {
			fmt.Printf("  - %s\n", target)
		}
	}

	return nil
}

